CONFIG = {
    "db": "/home/ubuntu/storage/neorefs/neoref16s/rev6/neoRef16S_rev6_190325.acc",
    "out_dir": "/home/ubuntu/storage/neorefs/genomes/bacteria",
    "email": "vinicius.salazar@biome-hub.com",
    "api_key": "afb857c557897af55345e77a84def712cd08",
}
